package com.jiuqi.office.excel;

import com.jiuqi.grid.CellField;
import com.jiuqi.grid.CurrencyCellProperty;
import com.jiuqi.grid.GridCell;
import com.jiuqi.grid.GridData;
import com.jiuqi.grid.GridFieldList;
import com.jiuqi.grid.NumberCellProperty;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.CellRangeAddress;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Color;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.RichTextString;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFRichTextString;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public final class WorksheetWriter {
   private Workbook workBook;
   private Sheet sheet;
   private GridData gridData;
   private String title;
   private boolean autoAdjust;
   private boolean addTitle;
   private Font titleFont;
   private HashMap cellStyleHashMap;
   private int transparentColor;
   private boolean doNotCheckNumberIsNull;
   private Map HSSFFontMap;
   private List sheets = new ArrayList();
   private static int MAX_COLUMN = 255;
   private String preSheetName;

   public WorksheetWriter(HSSFWorkbook workBook, HSSFSheet sheet, GridData gridData, HashMap cellStyleHashMap) {
      this.workBook = workBook;
      this.sheet = sheet;
      this.sheets.add(sheet);
      this.gridData = gridData;
      this.cellStyleHashMap = cellStyleHashMap;
      this.HSSFFontMap = new HashMap();
      this.transparentColor = -16777201;
   }

   public WorksheetWriter(Workbook workBook, Sheet sheet, GridData gridData, HashMap cellStyleHashMap) {
      this.workBook = workBook;
      this.sheet = sheet;
      this.sheets.add(sheet);
      this.gridData = gridData;
      this.cellStyleHashMap = cellStyleHashMap;
      this.HSSFFontMap = new HashMap();
      this.transparentColor = -16777201;
   }

   public String getPreSheetName() {
      return this.preSheetName;
   }

   public void setPreSheetName(String preSheetName) {
      this.preSheetName = preSheetName;
   }

   public void setTitle(String title) {
      this.title = title;
   }

   private void setDefaultTitleFont() {
      this.titleFont = this.workBook.createFont();
      this.titleFont.setFontName("宋体");
      this.titleFont.setBoldweight((short)700);
      this.titleFont.setFontHeightInPoints((short)12);
   }

   public void setAutoAdjust(boolean autoAdjust) {
      this.autoAdjust = autoAdjust;
   }

   public void setAddTitle(boolean addTitle) {
      this.addTitle = addTitle;
   }

   public void setTransparentColor(int transparentColor) {
      this.transparentColor = transparentColor;
   }

   public boolean isDoNotCheckNumberIsNull() {
      return this.doNotCheckNumberIsNull;
   }

   public void setDoNotCheckNumberIsNull(boolean doNotCheckNumberIsNull) {
      this.doNotCheckNumberIsNull = doNotCheckNumberIsNull;
   }

   public void writeWorkSheet() {
      this.addSheets();
      this.addTitle();
      this.writeContent();
      this.adjustHW();
      this.mergeCells();
      this.hideRowCol();
   }

   private Sheet getSheet(int index) {
      return (Sheet)this.sheets.get(index);
   }

   private int getSheetIndex(int col) {
      return col / MAX_COLUMN;
   }

   private int getCol(int col) {
      return col % MAX_COLUMN;
   }

   private void addSheets() {
      if (this.gridData.getColCount() > MAX_COLUMN) {
         int extraSheetsNum = (this.gridData.getColCount() + MAX_COLUMN - 1) / MAX_COLUMN - 1;

         for(int i = 0; i < extraSheetsNum; ++i) {
            this.sheets.add(this.workBook.createSheet(this.preSheetName + (i + 1)));
         }
      }

   }

   private void addTitle() {
      if (this.addTitle) {
         this.setDefaultTitleFont();
         RichTextString richText = this.getRichTextString(this.title, this.workBook);

         for(int i = 0; i < this.sheets.size(); ++i) {
            Cell cell = this.getSheet(i).createRow(0).createCell(0);
            if (richText != null) {
               cell.setCellValue(richText);
            }

            CellStyle titleCellStyle = this.workBook.createCellStyle();
            titleCellStyle.setAlignment((short)2);
            titleCellStyle.setFont(this.titleFont);
            cell.setCellStyle(titleCellStyle);
            this.getSheet(i).addMergedRegion(new CellRangeAddress(0, 0, 0, Math.min(this.gridData.getColCount() - 2, MAX_COLUMN)));
         }
      }

   }

   private void writeContent() {
      if (this.gridData == null) {
         throw new NullPointerException("表格数据不能为空");
      } else {
         int rowCount = this.gridData.getRowCount();
         int colCount = this.gridData.getColCount();

         for(int row = 1; row < rowCount; ++row) {
            for(int col = 1; col < colCount; ++col) {
               GridCell gridCell = this.gridData.getCell(col, row);
               this.writeCell(gridCell);
            }
         }

      }
   }

   private void writeCell(GridCell gridCell) {
      int rowNum = gridCell.getRowNum() - 1;
      int colNum = gridCell.getColNum() - 1;
      if (this.addTitle) {
         ++rowNum;
      }

      Row row = this.getSheet(this.getSheetIndex(colNum)).getRow(rowNum) == null ? this.getSheet(this.getSheetIndex(colNum)).createRow(rowNum) : this.getSheet(this.getSheetIndex(colNum)).getRow(rowNum);
      Cell cell = row.createCell(this.getCol(colNum));

      try {
         CellStyle cellStyle = this.buildCellStyle(gridCell);
         cell.setCellStyle(cellStyle);
         String cellData;
         RichTextString richTextString;
         double value;
         switch (gridCell.getDataType()) {
            case 0:
            case 1:
            case 4:
               cellData = gridCell.getCellData();
               RichTextString richTextString = this.getRichTextString(cellData, this.workBook);
               if (richTextString != null) {
                  cell.setCellValue(richTextString);
               }
               break;
            case 2:
               cellData = gridCell.getCellData();
               if (cellData != null && !cellData.equals("")) {
                  value = gridCell.getFloat();
                  cell.setCellValue(value);
                  if (gridCell.getShowText().endsWith("‰")) {
                     NumberCellProperty ncp = new NumberCellProperty(gridCell);
                     String decimalStr = this.buildDecimal(ncp, -3) + "‰";
                     NumberFormat numf = NumberFormat.getInstance();
                     numf.setGroupingUsed(true);
                     RichTextString richTextString = this.getRichTextString(numf.format(value * 1000.0), this.workBook);
                     if (richTextString == null) {
                        cell.setCellValue(richTextString + decimalStr);
                     } else {
                        cell.setCellValue(decimalStr);
                     }
                  }
               }
               break;
            case 3:
               cellData = gridCell.getCellData();
               if (cellData != null && !cellData.equals("")) {
                  if ("|".equals(cellData)) {
                     String value = gridCell.getShowText();
                     richTextString = this.getRichTextString(value, this.workBook);
                     if (richTextString != null) {
                        cell.setCellValue(richTextString);
                     }
                  } else {
                     value = Double.parseDouble(cellData);
                     cell.setCellValue(value);
                  }
               }
               break;
            case 5:
               Date date = gridCell.getDateTime();
               if (date != null) {
                  cell.setCellValue(date);
               }
            case 6:
            case 7:
            case 8:
            default:
               break;
            case 9:
               cellData = gridCell.getCellData();
               if (cellData != null && !"".equals(cellData)) {
                  String[] info = GridCell.parserLinkInformation(cellData);
                  richTextString = this.getRichTextString(info[0], this.workBook);
                  if (richTextString != null) {
                     cell.setCellValue(richTextString);
                  }
               }
         }
      } catch (Exception var14) {
         Exception e = var14;
         e.printStackTrace();
      }

   }

   private String makeCurFotmat(CurrencyCellProperty curCellProperty) {
      String currencyFormat = "#,##0";
      String colorStr = "";
      boolean isBracketNegative = false;
      currencyFormat = currencyFormat + this.buildDecimal(curCellProperty, 0);
      int idx = curCellProperty.getUnitIndex();
      if (curCellProperty.getWarningNegative()) {
         colorStr = "[Red]";
      }

      isBracketNegative = curCellProperty.getBracketNegative();
      String signStr;
      switch (curCellProperty.getUnitShowType()) {
         case 0:
            signStr = CurrencyCellProperty.CURRENCY_UNIT_SIGNS[idx];
            if (signStr.length() == 1) {
               currencyFormat = isBracketNegative ? signStr + currencyFormat + ";" + colorStr + signStr + "(" + currencyFormat + ")" : signStr + currencyFormat + ";" + colorStr + signStr + "-" + currencyFormat;
            } else if (signStr.length() > 1) {
               currencyFormat = isBracketNegative ? "[$" + signStr + "]" + currencyFormat + ";" + colorStr + "[$" + signStr + "](" + currencyFormat + ")" : "[$" + signStr + "]" + currencyFormat + ";" + colorStr + "[$" + signStr + "]-" + currencyFormat;
            }
            break;
         case 1:
            signStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
            currencyFormat = isBracketNegative ? "[$" + signStr + "] " + currencyFormat + ";" + colorStr + "[$" + signStr + "] (" + currencyFormat + ")" : "[$" + signStr + "] " + currencyFormat + ";" + colorStr + "[$" + signStr + "] -" + currencyFormat;
            break;
         case 2:
            signStr = CurrencyCellProperty.CURRENCY_UNIT_NAMES[idx];
            currencyFormat = isBracketNegative ? currencyFormat + " [$" + signStr + "];" + colorStr + "(" + currencyFormat + ") [$" + signStr + "]" : currencyFormat + " [$" + signStr + "];" + colorStr + "-" + currencyFormat + " [$" + signStr + "]";
            break;
         case 3:
            signStr = CurrencyCellProperty.CURRENCY_UNIT_TITLES[idx];
            currencyFormat = isBracketNegative ? "[$" + signStr + "] " + currencyFormat + ";" + colorStr + "[$" + signStr + "] (" + currencyFormat + ")" : "[$" + signStr + "] " + currencyFormat + ";" + colorStr + "[$" + signStr + "] -" + currencyFormat;
      }

      return currencyFormat;
   }

   private String buildDecimal(NumberCellProperty numberCellProperty, int offset) {
      int decimalCount = numberCellProperty.getDecimal() + offset;
      if (decimalCount <= 0) {
         return "";
      } else {
         String decimal = ".";

         for(int i = 0; i < decimalCount; ++i) {
            decimal = decimal + "0";
         }

         return decimal;
      }
   }

   private Font getCellFont(GridCell gridCell) {
      String fontKey = this.getCellFontKey(gridCell);
      Font font = (Font)this.HSSFFontMap.get(fontKey);
      if (font == null) {
         font = this.workBook.createFont();
         this.HSSFFontMap.put(fontKey, font);
         if (gridCell.getFontBold()) {
            font.setBoldweight((short)700);
         }

         Color fontColor = HSSFHelper.getApproximateTextColor(gridCell.getFontColor());
         if (fontColor != null) {
         }

         font.setFontHeight((short)gridCell.getFontHeight());
         font.setItalic(gridCell.getFontItalic());
         font.setFontName(gridCell.getFontName());
         font.setFontHeightInPoints((short)gridCell.getFontSize());
         font.setStrikeout(gridCell.getFontStrikeOut());
         if (gridCell.getFontUnderLine()) {
            font.setUnderline((byte)1);
         }
      }

      return font;
   }

   private String getCellFontKey(GridCell cell) {
      StringBuffer key = new StringBuffer();
      key.append(cell.getFontName());
      key.append(cell.getFontSize());
      key.append(cell.getFontColor());
      key.append(cell.getFontHeight());
      key.append(cell.getFontBold());
      key.append(cell.getFontItalic());
      key.append(cell.getFontStrikeOut());
      key.append(cell.getFontUnderLine());
      return key.toString();
   }

   private CellStyle buildCellStyle(GridCell gridCell) {
      CellStyle cellStyle = (CellStyle)this.cellStyleHashMap.get(gridCell);
      if (cellStyle != null) {
         return cellStyle;
      } else {
         GridCell tempGridCell = this.gridData.getCellEx(gridCell.getColNum(), gridCell.getRowNum());
         cellStyle = this.workBook.createCellStyle();
         short backColorIdx = 9;
         if (tempGridCell.getSilverHead()) {
            backColorIdx = 22;
         } else if (tempGridCell.getBackStyle() != 0) {
            HSSFColor backColor = HSSFHelper.getApproximateColor(tempGridCell.getBackColor());
            if (backColor != null) {
               backColorIdx = backColor.getIndex();
            }
         }

         cellStyle.setFillForegroundColor(backColorIdx);
         if (backColorIdx != 58) {
            cellStyle.setFillPattern((short)1);
         }

         short leftBorder = this.transBorderStyle(tempGridCell.getLEdgeStyle());
         short topBorder = this.transBorderStyle(tempGridCell.getTEdgeStyle());
         short rightBorder = this.transBorderStyle(tempGridCell.getREdgeStyle());
         short bottomBorder = this.transBorderStyle(tempGridCell.getBEdgeStyle());
         HSSFColor leftBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getLEdgeColor());
         HSSFColor topBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getTEdgeColor());
         HSSFColor righttBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getREdgeColor());
         HSSFColor bottomBorderColor = HSSFHelper.getApproximateEdgeColor(tempGridCell.getBEdgeColor());
         cellStyle.setBorderLeft(leftBorder);
         cellStyle.setBorderTop(topBorder);
         cellStyle.setBorderRight(rightBorder);
         cellStyle.setBorderBottom(bottomBorder);
         if (leftBorderColor != null) {
            cellStyle.setLeftBorderColor(leftBorderColor.getIndex());
         }

         if (topBorderColor != null) {
            cellStyle.setTopBorderColor(topBorderColor.getIndex());
         }

         if (righttBorderColor != null) {
            cellStyle.setRightBorderColor(righttBorderColor.getIndex());
         }

         if (bottomBorderColor != null) {
            cellStyle.setBottomBorderColor(bottomBorderColor.getIndex());
         }

         short horzAlign = this.transCellAlign(tempGridCell.getHorzAlign(), true);
         short vertAlign = this.transCellAlign(tempGridCell.getVertAlign(), false);
         cellStyle.setAlignment(horzAlign);
         cellStyle.setVerticalAlignment(vertAlign);
         cellStyle.setWrapText(tempGridCell.getWrapLine());
         cellStyle.setIndention((short)tempGridCell.getIndent());
         Font font = this.getCellFont(tempGridCell);
         cellStyle.setFont(font);
         short formatIndex;
         if (tempGridCell.getDataType() == 1) {
            formatIndex = this.workBook.createDataFormat().getFormat("@");
            cellStyle.setDataFormat(formatIndex);
         }

         String numStr;
         short formatIndex;
         if (tempGridCell.getDataType() == 2 && (this.doNotCheckNumberIsNull || tempGridCell.getShowText() != null)) {
            NumberCellProperty ncp;
            if (tempGridCell.getShowText() == null) {
               ncp = new NumberCellProperty(tempGridCell);
               numStr = "0";
               if (ncp.getThoundsMark()) {
                  numStr = "#,##0" + this.buildDecimal(ncp, 0);
               } else {
                  numStr = numStr + this.buildDecimal(ncp, 0);
               }

               formatIndex = this.workBook.createDataFormat().getFormat(numStr + " ");
               cellStyle.setDataFormat(formatIndex);
            } else if (tempGridCell.getShowText().endsWith("%")) {
               ncp = new NumberCellProperty(tempGridCell);
               numStr = "0" + this.buildDecimal(ncp, -2) + "%";
               formatIndex = this.workBook.createDataFormat().getFormat(numStr);
               cellStyle.setDataFormat(formatIndex);
            } else if (tempGridCell.getShowText().endsWith("‰")) {
               ncp = new NumberCellProperty(gridCell);
               numStr = "0";
               String colorStr = "";
               if (ncp.getThoundsMark()) {
                  numStr = "#,##0" + this.buildDecimal(ncp, 0);
               } else {
                  numStr = numStr + this.buildDecimal(ncp, 0);
               }

               if (ncp.getWarningNegative()) {
                  colorStr = "[Red]";
               }

               String formatStr = "";
               if (ncp.getBracketNegative()) {
                  formatStr = numStr + ";" + colorStr + "(" + numStr + ")";
               } else {
                  formatStr = numStr + ";" + colorStr + "-" + numStr;
               }

               short formatIndex = this.workBook.createDataFormat().getFormat(formatStr);
               cellStyle.setDataFormat(formatIndex);
            } else {
               ncp = new NumberCellProperty(tempGridCell);
               numStr = "0";
               if (ncp.getThoundsMark()) {
                  numStr = "#,##0" + this.buildDecimal(ncp, 0);
               } else {
                  numStr = numStr + this.buildDecimal(ncp, 0);
               }

               formatIndex = this.workBook.createDataFormat().getFormat(numStr + " ");
               cellStyle.setDataFormat(formatIndex);
            }
         }

         if (tempGridCell.getDataType() == 3) {
            CurrencyCellProperty curCellProperty = new CurrencyCellProperty(tempGridCell);
            numStr = this.makeCurFotmat(curCellProperty);
            formatIndex = this.workBook.createDataFormat().getFormat(numStr);
            cellStyle.setDataFormat(formatIndex);
         }

         if (gridCell.getDataType() == 5) {
            formatIndex = this.workBook.createDataFormat().getFormat("yyyy-m-d");
            cellStyle.setDataFormat(formatIndex);
         }

         gridCell.getDataType();
         this.cellStyleHashMap.put(tempGridCell, cellStyle);
         return cellStyle;
      }
   }

   private short transCellAlign(int align, boolean isHorizontal) {
      if (isHorizontal) {
         switch (align) {
            case 0:
               return 0;
            case 1:
               return 1;
            case 2:
               return 3;
            case 3:
               return 2;
            case 4:
            case 5:
               return 5;
            case 6:
               return 3;
            default:
               return 0;
         }
      } else {
         switch (align) {
            case 0:
               return 1;
            case 1:
               return 0;
            case 2:
               return 2;
            case 3:
            case 4:
            case 5:
               return 3;
            case 6:
               return 2;
            default:
               return 1;
         }
      }
   }

   private short transBorderStyle(int idx) {
      switch (idx) {
         case 0:
            return 1;
         case 1:
            return 0;
         case 2:
            return 1;
         case 3:
            return 3;
         case 4:
            return 7;
         case 5:
            return 9;
         case 6:
            return 11;
         case 7:
         case 8:
         case 9:
            return 5;
         case 10:
         case 11:
         case 12:
            return 6;
         case 13:
         case 14:
         case 15:
            return 5;
         default:
            return 1;
      }
   }

   private void mergeCells() {
      GridFieldList fieldList = this.gridData.merges();

      for(int i = 0; i < fieldList.count(); ++i) {
         CellField field = fieldList.get(i);
         int rowFrom = field.top - 1;
         int rowTo = field.bottom - 1;
         int colFrom = field.left - 1;
         int colTo = field.right - 1;
         if (this.addTitle) {
            ++rowFrom;
            ++rowTo;
         }

         int sheetFrom = this.getSheetIndex(colFrom);
         int sheetTo = this.getSheetIndex(colTo);

         for(int j = sheetFrom; j <= sheetTo; ++j) {
            int curColFrom = j == sheetFrom ? colFrom - j * MAX_COLUMN : 0;
            int curColTo = j == sheetTo ? colTo - j * MAX_COLUMN : MAX_COLUMN;
            this.getSheet(j).addMergedRegion(new CellRangeAddress(rowFrom, rowTo, curColFrom, curColTo));
         }
      }

   }

   private void adjustHW() {
      int colCount = this.gridData.getColCount();
      int rowCount = this.gridData.getRowCount();
      int row;
      if (this.autoAdjust) {
         for(row = 0; row < colCount; ++row) {
            this.getSheet(this.getSheetIndex(row)).autoSizeColumn((short)this.getCol(row));
         }
      } else {
         int i;
         for(row = 1; row < colCount; ++row) {
            i = this.gridData.getColWidths(row);
            this.getSheet(this.getSheetIndex(row)).setColumnWidth(this.getCol(row) - 1, HSSFHelper.pixelToWidth(i));
         }

         for(row = 1; row < rowCount; ++row) {
            if (this.gridData.getRowVisible(row)) {
               for(i = 0; i < this.sheets.size(); ++i) {
                  int rowHeight = this.gridData.getRowHeights(row);
                  Row hssfRow = this.addTitle ? this.getSheet(i).getRow(row) : this.getSheet(i).getRow(row - 1);
                  if (hssfRow != null) {
                     hssfRow.setHeight(HSSFHelper.pixelToHeight(rowHeight));
                  }
               }
            }
         }
      }

   }

   private void hideRowCol() {
      int row;
      for(row = 1; row < this.gridData.getColCount(); ++row) {
         if (!this.gridData.getColVisible(row)) {
            this.getSheet(this.getSheetIndex(row)).setColumnHidden(this.getCol(row) - 1, true);
         }
      }

      for(row = 1; row < this.gridData.getRowCount(); ++row) {
         if (!this.gridData.getRowVisible(row)) {
            for(int i = 0; i < this.sheets.size(); ++i) {
               Row hssfRow = this.addTitle ? this.getSheet(i).getRow(row) : this.getSheet(i).getRow(row - 1);
               hssfRow.setZeroHeight(true);
            }
         }
      }

   }

   public void setHSSFFontMap(Map hSSFFontMap) {
      this.HSSFFontMap = hSSFFontMap;
   }

   public Map getHSSFFontMap() {
      return this.HSSFFontMap;
   }

   private RichTextString getRichTextString(String title, Workbook wb) {
      RichTextString rt = null;
      if (wb instanceof HSSFWorkbook) {
         rt = new HSSFRichTextString(title);
      } else if (wb instanceof XSSFWorkbook) {
         rt = new XSSFRichTextString(title);
      } else if (wb instanceof SXSSFWorkbook) {
         rt = new XSSFRichTextString(title);
      }

      return (RichTextString)rt;
   }
}
